//
// "$Id: unittest_scrollbarsize.cxx 9479 2012-05-10 22:06:06Z matt $"
//
// Unit tests for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

#include "unittests.h"

#include <fltk3/Group.h>
#include <fltk3/Browser.h>
#include <fltk3/Tree.h>
#include <fltk3/ValueSlider.h>

//
// Test new 1.3.x global vs. local scrollbar sizing
//
class ScrollBarSizeTest : public fltk3::Group {
    fltk3::Browser *brow_a, *brow_b, *brow_c;
    fltk3::Tree    *tree_a, *tree_b, *tree_c;

    fltk3::Browser *makebrowser(int X,int Y,int W,int H,const char*L=0) {
	fltk3::Browser *b = new fltk3::Browser(X,Y,W,H,L);
	b->type(fltk3::MULTI_BROWSER);
	b->align(fltk3::ALIGN_TOP);
	b->add("Papa");     b->add("Delta"); b->add("Hotel");
        b->add("Long entry will show h-bar");
	b->add("Charlie");  b->add("Echo");  b->add("Foxtrot");
	b->add("Golf");     b->add("Lima");  b->add("Victor");
	b->add("Alpha");    b->add("Xray");  b->add("Yankee");
	b->add("Oscar");    b->add("India"); b->add("Juliet");
	b->add("Kilo");     b->add("Mike");  b->add("Sierra");
	b->add("November"); b->add("Tango"); b->add("Quebec");
	b->add("Bravo");    b->add("Romeo"); b->add("Uniform");
	b->add("Whisky");   b->add("Zulu");
	b->add("Papa");     b->add("Delta"); b->add("Hotel");
	b->add("Charlie");  b->add("Echo");  b->add("Foxtrot");
	b->add("Golf");     b->add("Lima");  b->add("Victor");
	b->add("Alpha");    b->add("Xray");  b->add("Yankee");
	b->add("Oscar");    b->add("India"); b->add("Juliet");
	b->add("Kilo");     b->add("Mike");  b->add("Sierra");
	b->add("November"); b->add("Tango"); b->add("Quebec");
	b->add("Bravo");    b->add("Romeo"); b->add("Uniform");
	b->add("Whisky");   b->add("Zulu");
	return(b);
    }
    fltk3::Tree *maketree(int X,int Y,int W,int H,const char*L=0) {
	fltk3::Tree *b = new fltk3::Tree(X,Y,W,H,L);
	b->type(fltk3::TREE_SELECT_MULTI);
	b->align(fltk3::ALIGN_TOP);
	b->add("Papa");     b->add("Delta"); b->add("Hotel");
        b->add("Long entry will show h-bar");
	b->add("Charlie");  b->add("Echo");  b->add("Foxtrot");
	b->add("Golf");     b->add("Lima");  b->add("Victor");
	b->add("Alpha");    b->add("Xray");  b->add("Yankee");
	b->add("Oscar");    b->add("India"); b->add("Juliet");
	b->add("Kilo");     b->add("Mike");  b->add("Sierra");
	b->add("November"); b->add("Tango"); b->add("Quebec");
	b->add("Bravo");    b->add("Romeo"); b->add("Uniform");
	b->add("Whisky");   b->add("Zulu");
	return(b);
    }
    void slide_cb2(fltk3::ValueSlider *in) {
	const char *label = in->label();
	int val = int(in->value());
	//fprintf(stderr, "VAL='%d'\n",val);
	if ( strcmp(label,"A: Scroll Size") == 0 ) {
	    brow_a->scrollbar_size(val);
	    tree_a->scrollbar_size(val);
	} else {
	    fltk3::scrollbar_size(val);
	}
	in->window()->redraw();
    }
    static void slide_cb(fltk3::Widget *w, void *data) {
        ScrollBarSizeTest *o = (ScrollBarSizeTest*)data;
	o->slide_cb2((fltk3::ValueSlider*)w);
    }
public: 
    static fltk3::Widget *create() {
      return(new ScrollBarSizeTest(TESTAREA_X, TESTAREA_Y, TESTAREA_W, TESTAREA_H));
    }

    // CTOR
    ScrollBarSizeTest(int X, int Y, int W, int H) : fltk3::Group(X,Y,W,H) {
      begin();
        //      _____________    _______________
        //     |_____________|  |_______________|
        //                                                ---   -----  <-- tgrpy
        //       brow_a      brow_b      brow_c            | 14   | 
        //     ----------  ----------  ----------         ---     |    <-- browy
        //     |        |  |        |  |        |          |      |
        //     |        |  |        |  |        |          |browh |
        //     |        |  |        |  |        |          |      |
        //     ----------  ----------  ----------         ---   tgrph 
        //                                                 |      |
        //       tree_a      tree_b      tree_c            | 20   | 
        //     ----------  ----------  ----------         ---     |    <-- treey
        //     |        |  |        |  |        |          |      |
        //     |        |  |        |  |        |          |treeh |
        //     |        |  |        |  |        |          |      |
        //     ----------  ----------  ----------         ---  ------
        //                                     
        int tgrpy = 30;
        int tgrph = H-130;
        int browy = tgrpy+14;
        int browh = tgrph/2 - 20;
        int treey = browy + browh + 20;
        int treeh = browh;
        brow_a = makebrowser( 10,browy,100,browh,"Browser A");
        brow_b = makebrowser(120,browy,100,browh,"Browser B");
        brow_c = makebrowser(240,browy,100,browh,"Browser C");
        tree_a = maketree( 10,treey,100,treeh,"Tree A");
        tree_b = maketree(120,treey,100,treeh,"Tree B");
        tree_c = maketree(240,treey,100,treeh,"Tree C");
        fltk3::ValueSlider *slide_glob = new fltk3::ValueSlider(100,0,100,18,"Global Scroll Size");
        slide_glob->value(16);
        slide_glob->type(fltk3::HORIZONTAL);
        slide_glob->align(fltk3::ALIGN_LEFT);
        slide_glob->range(0.0, 30.0);
        slide_glob->step(1.0);
        slide_glob->callback(slide_cb, (void*)this);
        slide_glob->labelsize(12);
        fltk3::ValueSlider *slide_browa = new fltk3::ValueSlider(350,0,100,18,"A: Scroll Size");
        slide_browa->value(16);
        slide_browa->type(fltk3::HORIZONTAL);
        slide_browa->align(fltk3::ALIGN_LEFT);
        slide_browa->range(0.0, 30.0);
        slide_browa->step(1.0);
        slide_browa->callback(slide_cb, (void*)this);
        slide_browa->labelsize(12);
      end();
      label("Verify global scroll sizing and per-widget scroll sizing.\n"
	    "Scrollbar's size should change interactively as size sliders are changed.\n"
            "Changing 'Global Scroll Size' should affect all three browser's scrollbars UNLESS\n"
	    "the 'A: Scroll Size' slider is changed, in which case its value will take precedence\n"
	    "for 'Browser A', and the global size will only affect Browser B and C.");
      labelsize(12);
      align(fltk3::ALIGN_INSIDE|fltk3::ALIGN_BOTTOM|fltk3::ALIGN_LEFT|fltk3::ALIGN_WRAP);
    }
};

UnitTest scrollbarsize("scrollbar size", ScrollBarSizeTest::create);

//
// End of "$Id: unittest_scrollbarsize.cxx 9479 2012-05-10 22:06:06Z matt $"
//
